@@ -10,7 +10,7 @@ if ($@) {
     @ISA = qw(CGI::XA);
 }
 
-$VERSION = (qw$Revision: 1.17 $)[1];
+$VERSION = (qw$Revision: 1.20 $)[1];
 
 sub new {
     my($class) = shift;
@@ -25,7 +25,7 @@ sub header {
     my $self = shift;
     my $r = $self->{'.req'};
     $r->basic_http_header;
-    return $self->CGI::XA::header(@hdrs);
+    return $self->CGI::XA::header(@_);
 }		     
 
 sub print {
@@ -48,11 +48,11 @@ sub new_MultipartBuffer {
 
 sub exit {
     my($self, $s) = @_;
-    $self->{'.req'}->connection->close;
     $self->{'.req'}->exit($s);
 }
 
 package Apache::MultipartBuffer;
+use vars qw(@ISA);
 @ISA = qw(CGI::XA::MultipartBuffer);
 
 sub wouldBlock { undef }
